
#include "vusbdevice.h"

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <time.h>

#include <gtk/gtk.h>


#define USBDEV_SHARED_VENDOR    0x16C0  /* VOTI */
#define USBDEV_SHARED_PRODUCT   0x05DC  /* Obdev's free shared PID */
/* Use obdev's generic shared VID/PID pair and follow the rules outlined
 * in firmware/usbdrv/USBID-License.txt.
 */

#define USBDEV_VENDOR_NAME	"www.matrixstorm.com"
#define USBDEV_DEVICE_NAME	"USB<->PC"
#define USBDEV_SERIAL		"template"

/* USB Request-Commands */
#define PSCMD_CALL_ECHO		0
#define PSCMD_CALL_GETSTATUS	1

/* USB Response-Codes */
#define PSCMD_RETN_OKAY		(0x0)
#define PSCMD_RETN_BUSY		(0x1)
#define PSCMD_RETN_UNKNOW	(0xff)



static usb_dev_handle	*handle		= NULL;
static GtkBuilder	*builder	= NULL;

#define MYDEBUG(msg...) ({fprintf(stderr,"%s:%i (%s)   ",__FILE__,__LINE__, __FUNCTION__); fprintf(stderr, ##msg); })

static gboolean textview_add(GtkWidget *widget, char *text) {
  GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
  if (buffer) {
    GtkTextIter iter;
    gtk_text_buffer_get_end_iter(buffer, &iter);
    gtk_text_buffer_insert(buffer, &iter, text, strlen(text));
    gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW (widget), &iter, 0.0, TRUE, 0.0, 0.0);
  }
  return TRUE;
}

static gboolean textview1_add(char *text) {
  GtkWidget  *textview1 = GTK_WIDGET (gtk_builder_get_object (builder, "textview1"));
  textview_add(textview1, text);
}

G_MODULE_EXPORT void on_button1_clicked(GtkWidget *widget, gpointer data) {
  if (handle) {
    char		textline[256];
    unsigned char      buffer[32 + 1 + 1];
    int                nBytes;
    int			i, v, r;

    /* The test consists of writing 1000 random numbers to the device and checking
     * the echo. This should discover systematic bit errors (e.g. in bit stuffing).
     */
    for(i=0;i<1000;i++){
      v = rand() & 0xffff;
      nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, PSCMD_CALL_ECHO, v, 0, (char *)buffer, sizeof(buffer), 5000);
      if(nBytes < 4){
	  if(nBytes < 0) snprintf(textline,sizeof(textline)-1, "USB error: %s\n", usb_strerror());
	  snprintf(textline, sizeof(textline)-1, "only %d bytes received in iteration %d\nvalue sent = 0x%x\n", nBytes, i, v);
	  textview1_add(textline);
	  return;
      } 
      if (buffer[0] != PSCMD_CALL_ECHO) {
	  snprintf(textline, sizeof(textline)-1, "data error: received wrong command 0x%x instead of 0x%x in iteration %d\n", (int)buffer[0], PSCMD_CALL_ECHO, i);
	  textview1_add(textline);
	  return;
      }
      if (buffer[1] != PSCMD_RETN_OKAY) {
	  snprintf(textline, sizeof(textline)-1, "data error: received wrong return 0x%x instead of 0x%x in iteration %d\n", (int)buffer[1], PSCMD_RETN_OKAY, i);
	  textview1_add(textline);
	  return;
      }
      r = buffer[2] | (buffer[3] << 8);
      if(r != v){
	  snprintf(textline, sizeof(textline)-1, "data error: received 0x%x instead of 0x%x in iteration %d\n", r, v, i);
	  textview1_add(textline);
	  return;
      }
    }
    snprintf(textline, sizeof(textline)-1, "test succeeded\n");
    textview1_add(textline);
  }
}

G_MODULE_EXPORT void on_button2_clicked(GtkWidget *widget, gpointer data) {

  if (handle) {
    char		 textline[256];
    unsigned char       buffer[32 + 1 + 1];
    int                 nBytes;

    nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, PSCMD_CALL_GETSTATUS, 0, 0, (char *)buffer, sizeof(buffer), 5000);

    if (nBytes >= 8) {
      int	i;
      uint64_t	jiffies = 0;

      for (i=7;i>2;i--) {
	jiffies <<=8;
	jiffies  |=buffer[i];
      }
      snprintf(textline, sizeof(textline)-1, "internal state: %i ,  jiffies=%llu\n",(int)buffer[2],(unsigned long long int)jiffies);
      textview1_add(textline);
    } else {
      snprintf(textline, sizeof(textline)-1, "too short return\n");
      textview1_add(textline);
    }

  }

}


G_MODULE_EXPORT void on_window1_destroy(GtkWidget *widget, gpointer data) {
  gtk_main_quit ();
}

G_MODULE_EXPORT gboolean on_window1_delete_event(GtkWidget *widget, GdkEvent  *event, gpointer data) {
  return FALSE;
}

int main (int argc, char *argv[]) {
  srand(time(NULL));
  vusbdevice_initialize();
  
  gtk_init(&argc, &argv);

  if (g_module_supported()) {
    builder = gtk_builder_new ();
    if (builder) {
      if (gtk_builder_add_from_file (builder, "main.glade", NULL)) {
	GtkWidget  *window1 = GTK_WIDGET (gtk_builder_get_object (builder, "window1"));;
  
	gtk_builder_connect_signals (builder, NULL);                 
	gtk_widget_show_all (window1);

	if(usbOpenDevice(&handle, USBDEV_SHARED_VENDOR, USBDEV_SHARED_PRODUCT, USBDEV_VENDOR_NAME, USBDEV_DEVICE_NAME, USBDEV_SERIAL) != 0) {
	  if (handle) usb_close(handle);
	  handle=NULL;
	  textview1_add("usbOpenDevice error\n");
	}

	gtk_main ();
      } else MYDEBUG("reading file failed\n");
    } else MYDEBUG("out of memory?\n");
  } else MYDEBUG("g_module is NOT supported\n");

  if (builder) g_object_unref (G_OBJECT (builder));
  builder=NULL;
  if (handle) usb_close(handle);
  handle=NULL;
  vusbdevice_finalize();
  return 0;
}
