
#include "vusbdevice.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <time.h>

#define USBDEV_SHARED_VENDOR    0x16C0  /* VOTI */
#define USBDEV_SHARED_PRODUCT   0x05DC  /* Obdev's free shared PID */
/* Use obdev's generic shared VID/PID pair and follow the rules outlined
 * in firmware/usbdrv/USBID-License.txt.
 */

#define USBDEV_VENDOR_NAME	"www.matrixstorm.com"
#define USBDEV_DEVICE_NAME	"USB<->PC"
#define USBDEV_SERIAL		"template"

/* USB Request-Commands */
#define PSCMD_CALL_ECHO		0
#define PSCMD_CALL_GETSTATUS	1

/* USB Response-Codes */
#define PSCMD_RETN_OKAY		(0x0)
#define PSCMD_RETN_BUSY		(0x1)
#define PSCMD_RETN_UNKNOW	(0xff)

static void usage(char *name) {
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "  %s test\n\n", name);
    fprintf(stderr, "  %s status\n\n", name);
}



int main(int argc, char **argv) {
  usb_dev_handle      *handle = NULL;
  unsigned char       buffer[32 + 1 + 1];
  int                 nBytes;

  if(argc < 2){
      usage(argv[0]);
      exit(1);
  }

  srand(time(NULL));
  vusbdevice_initialize();

  if(usbOpenDevice(&handle, USBDEV_SHARED_VENDOR, USBDEV_SHARED_PRODUCT, USBDEV_VENDOR_NAME, USBDEV_DEVICE_NAME, USBDEV_SERIAL) != 0) {
      fprintf(stderr, "Could not find USB device \"USBTest\" with vid=0x%x pid=0x%x\n", USBDEV_SHARED_VENDOR, USBDEV_SHARED_PRODUCT);
      exit(1);
  }

  /* We have searched all devices on all busses for our USB device above. Now
   * try to open it and perform the vendor specific control operations for the
   * function requested by the user.
   */
  if (strcmp(argv[1], "test") == 0) {
    int i, v, r;

    /* The test consists of writing 1000 random numbers to the device and checking
     * the echo. This should discover systematic bit errors (e.g. in bit stuffing).
     */
    for(i=0;i<1000;i++){
      v = rand() & 0xffff;
      nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, PSCMD_CALL_ECHO, v, argc>2?16384:0, (char *)buffer, sizeof(buffer), 5000);
      if(nBytes < 4){
	  if(nBytes < 0) fprintf(stderr, "USB error: %s\n", usb_strerror());
	  fprintf(stderr, "only %d bytes received in iteration %d\n", nBytes, i);
	  fprintf(stderr, "value sent = 0x%x\n", v);
	  exit(1);
      } 
      if (buffer[0] != PSCMD_CALL_ECHO) {
	  fprintf(stderr, "data error: received wrong command 0x%x instead of 0x%x in iteration %d\n", (int)buffer[0], PSCMD_CALL_ECHO, i);
	  exit(1);
      }
      if (buffer[1] != PSCMD_RETN_OKAY) {
	  fprintf(stderr, "data error: received wrong return 0x%x instead of 0x%x in iteration %d\n", (int)buffer[1], PSCMD_RETN_OKAY, i);
	  exit(1);
      }
      r = buffer[2] | (buffer[3] << 8);
      if(r != v){
	  fprintf(stderr, "data error: received 0x%x instead of 0x%x in iteration %d\n", r, v, i);
	  exit(1);
      }
    }
    printf("test succeeded\n");
  } else if(strcmp(argv[1], "status") == 0) {
    nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, PSCMD_CALL_GETSTATUS, 0, 0, (char *)buffer, sizeof(buffer), 5000);
    if (nBytes >= 8) {
      int i;
      uint64_t jiffies = 0;
      for (i=7;i>2;i--) {
	jiffies <<=8;
	jiffies  |=buffer[i];
      }
      fprintf(stdout,"internal state: %i ,  jiffies=%llu\n",(int)buffer[2],(unsigned long long int)jiffies);
    } else fprintf(stderr,"too short return\n");
  } else {
    usage(argv[0]);
    exit(1);
  }

  if (handle) usb_close(handle);
  vusbdevice_finalize();
  return 0;
}
