unit UsbUnit;

interface

const
  LIBUSB_PATH_MAX = 512;

  USB_OK = 0; // 0 = success from functions < 0 is failed


  { Device and/or Interface Class codes }
  USB_CLASS_PER_INTERFACE   =   0; { for DeviceClass }
  USB_CLASS_AUDIO           =   1;
  USB_CLASS_COMM            =   2;
  USB_CLASS_HID             =   3;
  USB_CLASS_PRINTER         =   7;
  USB_CLASS_MASS_STORAGE    =   8;
  USB_CLASS_HUB             =   9;
  USB_CLASS_DATA            =  10;
  USB_CLASS_VENDOR_SPEC     = $ff;

  // Descriptor types
  USB_DT_DEVICE     = $01;
  USB_DT_CONFIG     = $02;
  USB_DT_STRING     = $03;
  USB_DT_INTERFACE  = $04;
  USB_DT_ENDPOINT   = $05;

  USB_DT_HID        = $21;
  USB_DT_REPORT     = $22;
  USB_DT_PHYSICAL   = $23;
  USB_DT_HUB        = $29;

  // Descriptor sizes per descriptor type
  USB_DT_DEVICE_SIZE          = 18;
  USB_DT_CONFIG_SIZE          =  9;
  USB_DT_INTERFACE_SIZE       =  9;
  USB_DT_ENDPOINT_SIZE        =  7;
  USB_DT_ENDPOINT_AUDIO_SIZE  =  9; // Audio extension
  USB_DT_HUB_NONVAR_SIZE      =  7;


type
  // All standard descriptors have these 2 fields in common
  usb_descriptor_header = packed record
    bLength,
    bDescriptorType: byte;
  end;

  // String descriptor
  usb_string_descriptor = packed record
    bLength,
    bDescriptorType: byte;
    wData: packed array [0..0] of word;
  end;

  usb_hid_descriptor = packed record
    bLength,
    bDescriptorType: byte;
    bcdHID: word;
    bCountryCode,
    bNumDescriptors: byte;
  end;

const
  // Endpoint descriptor
  USB_MAXENDPOINTS = 32;

type
  // Endpoint descriptor
  pusb_endpoint_descriptor = ^usb_endpoint_descriptor;
  usb_endpoint_descriptor = packed record
    bLength,
    bDescriptorType,
    bEndpointAddress,
    bmAttributes: byte;

    wMaxPacketSize: word;

    bInterval,
    bRefresh,
    bSynchAddress: byte;

    extra: PByte; // Extra descriptors
    extralen: integer;
  end;
  // pascal translation of C++ struc
  TArray_usb_endpoint_descriptor = packed array [0..65535] of usb_endpoint_descriptor;
  PArray_usb_endpoint_descriptor = ^TArray_usb_endpoint_descriptor;


const
  USB_ENDPOINT_ADDRESS_MASK      = $0f; // in bEndpointAddress
  USB_ENDPOINT_DIR_MASK          = $80;

  USB_ENDPOINT_TYPE_MASK         = $03; // in bmAttributes
  USB_ENDPOINT_TYPE_CONTROL      = 0;
  USB_ENDPOINT_TYPE_ISOCHRONOUS  = 1;
  USB_ENDPOINT_TYPE_BULK         = 2;
  USB_ENDPOINT_TYPE_INTERRUPT    = 3;

  // Interface descriptor
  USB_MAXINTERFACES = 32;

type
  // Interface descriptor
  pusb_interface_descriptor = ^usb_interface_descriptor;
  usb_interface_descriptor = packed record
    bLength,
    bDescriptorType,
    bInterfaceNumber,
    bAlternateSetting,
    bNumEndpoints,
    bInterfaceClass,
    bInterfaceSubClass,
    bInterfaceProtocol,
    iInterface: byte;

    endpoint: PArray_usb_endpoint_descriptor;

    extra: PByte; // Extra descriptors
    extralen: integer;
  end;
  // pascal translation of C++ struc
  TArray_usb_interface_descriptor = packed array [0..65535] of usb_interface_descriptor;
  PArray_usb_interface_descriptor = ^TArray_usb_interface_descriptor;


const
  USB_MAXALTSETTING = 128; // Hard limit

type
  pusb_interface = ^usb_interface;
  usb_interface = packed record
    altsetting: PArray_usb_interface_descriptor;
    num_altsetting: integer;
  end;
  // pascal translation of C++ struc
  TArray_usb_interface = packed array [0..65535] of usb_interface;
  PArray_usb_interface = ^TArray_usb_interface;


const
  // Configuration descriptor information..
  USB_MAXCONFIG = 8;

type
  // Configuration descriptor information..
  pusb_config_descriptor = ^usb_config_descriptor;
  usb_config_descriptor = packed record
    bLength,
    bDescriptorType: byte;

    wTotalLength: word;

    bNumInterfaces,
    bConfigurationValue,
    iConfiguration,
    bmAttributes,
    MaxPower: byte;

    iinterface: PArray_usb_interface;

    extra: PByte; // Extra descriptors
    extralen: integer;
  end;
  // pascal translation of C++ struc
  TArray_usb_config_descriptor = packed array [0..65535] of usb_config_descriptor;
  PArray_usb_config_descriptor = ^TArray_usb_config_descriptor;


  // Device descriptor
  usb_device_descriptor = packed record
    bLength,
    bDescriptorType: byte;
    bcdUSB: word;

    bDeviceClass,
    bDeviceSubClass,
    bDeviceProtocol,
    bMaxPacketSize0: byte;

    idVendor,
    idProduct,
    bcdDevice: word;

    iManufacturer,
    iProduct,
    iSerialNumber,
    bNumConfigurations: byte;
  end;

  usb_ctrl_setup = packed record
    bRequestType,
    bRequest: byte;
    wValue,
    wIndex,
    wLength: word;
  end;

const
  // Standard requests
  USB_REQ_GET_STATUS         = $00;
  USB_REQ_CLEAR_FEATURE      = $01;
  // $02 is reserved
  USB_REQ_SET_FEATURE        = $03;
  // $04 is reserved
  USB_REQ_SET_ADDRESS        = $05;
  USB_REQ_GET_DESCRIPTOR     = $06;
  USB_REQ_SET_DESCRIPTOR     = $07;
  USB_REQ_GET_CONFIGURATION  = $08;
  USB_REQ_SET_CONFIGURATION  = $09;
  USB_REQ_GET_INTERFACE      = $0A;
  USB_REQ_SET_INTERFACE      = $0B;
  USB_REQ_SYNCH_FRAME        = $0C;

  USB_TYPE_STANDARD   = ($00 shl 5);
  USB_TYPE_CLASS      = ($01 shl 5);
  USB_TYPE_VENDOR     = ($02 shl 5);
  USB_TYPE_RESERVED   = ($03 shl 5);

  USB_RECIP_DEVICE     = $00;
  USB_RECIP_INTERFACE  = $01;
  USB_RECIP_ENDPOINT   = $02;
  USB_RECIP_OTHER      = $03;

  // Various libusb API related stuff
  USB_ENDPOINT_IN   = $80;
  USB_ENDPOINT_OUT  = $00;

  // Error codes
  USB_ERROR_BEGIN  = 500000;

type
  pusb_device = ^usb_device;
  pusb_bus = ^usb_bus;

  usb_device = packed record
    next,
    prev: pusb_device;
    filename: packed array [0..LIBUSB_PATH_MAX-1] of char;
    bus: pusb_bus;
    descriptor: usb_device_descriptor;
    config: PArray_usb_config_descriptor;

    dev: pointer; // Darwin support
  end;

  usb_bus = packed record
    next,
    prev: pusb_bus;
    dirname: packed array [0..LIBUSB_PATH_MAX-1] of char;
    devices: pusb_device;
    location: longint;
   end;

  // Version information, Windows specific
  pusb_version = ^usb_version;
  usb_version = packed record
    dllmajor,
    dllminor,
    dllmicro,
    dllnano: integer;
    drivermajor,
    driverminor,
    drivermicro,
    drivernano: integer;
  end;  

  pusb_dev_handle = pointer; // struct usb_dev_handle;
 // typedef struct usb_dev_handle usb_dev_handle;

procedure usb_init; stdcall;
function usb_find_busses : integer; stdcall;
function usb_find_devices : integer; stdcall;
function usb_get_busses : pusb_bus; stdcall;
function usb_control_msg(dev: pusb_dev_handle;requesttype, request, value, index: integer;var bytes;size, timeout: integer): integer; stdcall;
function usb_open(dev : pusb_device) : pusb_dev_handle; stdcall;
function usb_close(dev : pusb_dev_handle) : integer; stdcall;



const
      USB_ERROR_NOTFOUND = 1;
      USB_ERROR_ACCESS   = 2;
      USB_ERROR_IO       = 3;
function usbGetStringAscii(dev : pusb_dev_handle; index : integer; langid : integer; var outputbuffer; buflen : integer) : integer;
function usbOpenDevice(var device : pusb_dev_handle; vendor : word; product : word; vendorName : pchar; productName : pchar; serial : pchar) : integer;

implementation
uses SysUtils;

procedure usb_init; stdcall; external 'libusb0.dll' name 'usb_init';
function usb_find_busses : integer; stdcall; external 'libusb0.dll' name 'usb_find_busses';
function usb_find_devices : integer; stdcall; external 'libusb0.dll' name 'usb_find_devices';
function usb_get_busses : pusb_bus; stdcall; external 'libusb0.dll' name 'usb_get_busses';
function usb_control_msg(dev: pusb_dev_handle;requesttype, request, value, index: integer;var bytes;size, timeout: integer): integer; stdcall; external 'libusb0.dll' name 'usb_control_msg';
function usb_open(dev : pusb_device) : pusb_dev_handle; stdcall; external 'libusb0.dll' name 'usb_open';
function usb_close(dev : pusb_dev_handle) : integer; stdcall; external 'libusb0.dll' name 'usb_close';



function usbGetStringAscii(dev : pusb_dev_handle; index : integer; langid : integer; var outputbuffer; buflen : integer) : integer;
type   usbGetStringAsciiBuffer = array[0..$ffff] of char;
       PusbGetStringAsciiBuffer = ^usbGetStringAsciiBuffer;
var
    buffer   : array[0..255] of char;
    rval, i  : integer;

    buf      : PusbGetStringAsciiBuffer;

begin
  buf:=addr(outputbuffer);
  buffer:='test';
  rval:=usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, (USB_DT_STRING shl 8) + index, langid, buffer, sizeof(buffer), 1000);

  if (rval < 0) then
    begin
      result:=rval;
      exit;
    end;

  if (ord(buffer[1]) <> USB_DT_STRING) then
    begin
      result:=0;
      exit;
    end;

  if (ord(buffer[0]) < rval) then rval:=ord(buffer[0]);

  rval:=rval shr 1;
  i:=1;
  while (i<rval) do
    begin
      if (i > buflen) then break;
      buf[i-1] := buffer[i shl 1];
      if (ord(buffer[(i shl 1)+1]) <> 0) then buf[i-1]:='?';
      inc(i);
    end;
  buf[i-1]:=chr(0);
  result:=i-1;
end;

function usbOpenDevice(var device : pusb_dev_handle; vendor : word; product : word; vendorName : pchar; productName : pchar; serial : pchar) : integer;
var
  bus       : pusb_bus;
  dev       : pusb_device;
  handle    : pusb_dev_handle;
  errorcode : integer;

  usbstring : array[0..255] of char;
  len       : integer;
  


begin
  errorcode:=USB_ERROR_NOTFOUND;
  usb_find_busses;
  usb_find_devices;

 bus:=usb_get_busses;
 while (bus <> NIL) do
 begin
  dev:=bus^.devices;
  while (dev <> NIL) do
  begin
    handle:=usb_open(dev);
    if (handle <> NIL) then
      begin
        if ((vendorName=NIL) and (productName=NIL)) then break;

        len := usbGetStringAscii(handle, dev^.descriptor.iManufacturer, $0409, usbstring, sizeof(usbstring));
        if (len >= 0) then
          begin
            errorcode:=USB_ERROR_NOTFOUND;
            if (StrComp(vendorName, usbstring)=0) then
              begin // found the matching vendor
                len := usbGetStringAscii(handle, dev^.descriptor.iProduct, $0409, usbstring, sizeof(usbstring));
                if (len >= 0) then
                 begin
                  errorcode:=USB_ERROR_NOTFOUND;
                  if (StrComp(productName, usbstring)=0) then
                    begin // found the matching product

                     len := usbGetStringAscii(handle, dev^.descriptor.iSerialNumber, $0409, usbstring, sizeof(usbstring));
                     if (len >= 0) then
                      begin
                        errorcode:=USB_ERROR_NOTFOUND;
                        if (StrComp(serial, usbstring)=0) then
                          begin // found also the right serial
                            break;
                          end;
                      end else errorcode:=USB_ERROR_IO;

                    end;
                 end  else errorcode:=USB_ERROR_IO;
              end;
          end         else errorcode:=USB_ERROR_IO;

        usb_close(handle);
        handle:=NIL;
      end             else errorcode:=USB_ERROR_ACCESS;
    dev:=dev^.next;
  end;
  if (handle<>NIL) then break;
  bus:=bus^.next;
 end;

 if (handle<>NIL) then
  begin
    device:=handle;
    result:=0;
  end             else result:=errorcode;
end; 

end.
